package org.illeva.jordino.answer;

import java.io.*;
import javax.mail.*;
import javax.mail.internet.*;

/**
  * This is the ancestor for all answer.
  *
  * @version Jordino V1.0
  * @author IlLeva
  */
abstract class Response
{
  // Data
  private MimeMessage dInMsg;
  private MimeMessage dOutMsg;
  protected int dStatus=0;
  /**
    * This is the buffer to add the message text.
    * 
    * @since Version 1.0
    */
  protected StringBuffer dBuff=new StringBuffer();
  protected String dUser="";
  protected String dPassword="";
  protected String dAction="";
  // Methods
  /**
    * Extract the part of subject that is the command to Jordino!
    * @return  The command in the subject of message.
    *
    * @since Version 1.0
    */
  public String getCommand()
  {
    String vRc=null;
    try
    {
      vRc=dInMsg.getSubject();
      if(vRc!=null)
      {
        vRc=vRc.substring(vRc.indexOf("[JD")+3);
      } // end if
    } // end try
    catch(MessagingException anEx)
    {
      System.err.println(anEx);
    } // end catch
    return vRc;
  } // end getCommand
  /**
    * Return the object status. If value is 0 then the object is Ok!
    * @return Status of object.
    *
    * @since Version 1.0
    */
  public int getStatus() {return dStatus;}
  /**
    * Base constructor for all answer message.
    * This class use the original message to construct the destination,
    * the subjet and the body of answer.
    * @param aMsg - Source Message.
    *
    * @since Version 1.0
    */
  protected Response(MimeMessage aMsg)
  {
    dInMsg=aMsg;
    parseText();
    try
    {
      dOutMsg=(MimeMessage)aMsg.reply(false);
      translateSubject();
      translateTo();
      translateFrom();
      dOutMsg.setContent(buildMessage());
    } // end try
    catch(MessagingException anEx)
    {
      dStatus=1;
      System.err.println(anEx);
    } // end catch
  } // end constructor
  /**
    * By default the receiver of  response is the sender of answer.
    * In the base class this method is empty.
    *
    * @since Version 1.0
    */
  protected void translateTo() throws MessagingException 
  {
  } // end translateTo
  /**
    * By default the sender is the receiver of answer.
    * In the base class this method is empty.
    *
    * @since Version 1.0
    */
  protected void translateFrom() throws MessagingException 
  {
  } // end translateFrom
  /**
    * Build the subject of answer removing the Jordino command at the and of
    * original subject and prefix itself with "jdR: " string.
    *
    * @since Version 1.0
    */
  protected void translateSubject() throws MessagingException 
  {
    String vTemp=dInMsg.getSubject();
    int vPos=vTemp.indexOf("[JD");
    if(vPos!=-1)
      vTemp=vTemp.substring(vPos+4);
    vTemp=vTemp.trim();
    vTemp="RJD: "+vTemp;
    dOutMsg.setSubject(vTemp);
  } // end translateSubject
  /**
    * Puts an header to body text.
    *
    * @since Version 1.0
    */
  protected void prepareTextHeader()
  {
    dBuff.append("Jordino V1.0 - Automatic Responder Mailer\n");
  } // end prepareTextHeader
  /**
    * Puts a foother to body text.
    *
    * @since Version 1.0
    */
  protected void prepareTextFooter()
  {
    dBuff.append("Bye :)\n");
    dBuff.append("Jordino.");
  } // end prepareTextFooter
  abstract protected void prepareTextBody();
  /**
    * Build the text message whith header, body and footer part.
    *
    * @since Version 1.0
    */
  protected MimeBodyPart addTextPart()
  {
    MimeBodyPart vTextPart=null;
    try
    {
      vTextPart=new MimeBodyPart();
     prepareTextHeader();
     if(checkAuthorization())
      prepareTextBody();
     else
      noGrant();
      prepareTextFooter();
      vTextPart.setText(dBuff.toString());
    } // end try
    catch(MessagingException anEx)
    {
      System.err.println(anEx);
    } // end catch
    return vTextPart;
  } // end addTextPart
  /**
    * This method create a MimeMultipart object and adds to it the text part of message.
    *
    * @since Version 1.0
    */
  protected Multipart buildMessage()
  {
    Multipart vMultiPart=null;
    try
    {
      vMultiPart= new MimeMultipart();
      vMultiPart.addBodyPart(addTextPart());
    }
    catch(MessagingException anEx)
    {
      System.err.println(anEx);
    } // end catch
    return vMultiPart;
  } // end buildMessage
  /**
    * This method return the answer message if the status object is set to 0.
    * @return The answer's message. Null, if some problem is encuntered.
    *
    * @since Version 1.0
    */
  MimeMessage getMessage()
  {
    MimeMessage vRc=null;
    if(dStatus==0)
      vRc=dOutMsg;
    return vRc;
  } // end getMessage
  /**
    * Return the string rappresentation of permission to perform the specific response.
    * The default value is "PUBLIC" that means no grants are required.
    * @since Version 1.0
    */
  protected String getOperationPermission()
  {
    return "PUBLIC";
  } // end method
  /**
    * Return true if the user type the correct password and he is authorized to 
    * perform request action.
    * If the permission of action is PUBLIC this method returns true too.
    * @since Version 1.0
    */
  private boolean checkAuthorization()
  {
    boolean vRc=false;
    if(getOperationPermission().equals("PUBLIC"))
      vRc=true;
    else if(!dUser.equals("")&&!dPassword.equals(""))
    {
      vRc=checkUserPassword()&&checkUserPermission();
    } // end else if
    return vRc;
  } // end method
  private void noGrant()
  {
    dBuff.append("\nSorry, you are no granted\n");
    dBuff.append("to perform this operation!\n\n");
  } // end method
  /**
    * Return true if the user can perform the operations.
    * @since Version 1.0
    */
  protected boolean checkUserPermission()
  {
    return false;
  } // end method
  protected boolean checkUserPassword()
  {
    return false;
  } // end method
  protected String parseEngine(String aLine)
  {
    String vRc=null;
    if(aLine!=null)
    {
      aLine=aLine.trim();
      if(aLine.startsWith("Action("))
      {
      } // end if
      else if(aLine.startsWith("User("))
      {
      } // end else if
      else if(aLine.startsWith("Password("))
      {
      } // end else if
      else
        vRc=aLine;
     } // end if
    return vRc;
  } // end method
  private void parseText()
  {
    try
    {
      BufferedReader vBuff=new BufferedReader(new InputStreamReader(dInMsg.getInputStream()));
      String vLine=vBuff.readLine();
      while(vLine!=null)
      {
        parseEngine(vLine);
        vLine=vBuff.readLine();
      } // end while
    } // end try
    catch(MessagingException anEx)
    {
      System.err.println(anEx);
    } // end catch
    catch(IOException anEx)
    {
      System.err.println(anEx);
    } // end catch
  } // end method
} // end class